package geniusweb.protocol.session.mopac;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.Action;
import geniusweb.actions.Offer;
import geniusweb.actions.PartyId;
import geniusweb.inform.Inform;
import geniusweb.protocol.partyconnection.ProtocolToPartyConn;
import geniusweb.protocol.partyconnection.ProtocolToPartyConnFactory;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.repository.NoResourcesNowException;

public class MOPACTest {

	private final ObjectMapper jackson = new ObjectMapper();

	private static final PartyId PARTY1ID = new PartyId("party1");
	private static final PartyId PARTY2ID = new PartyId("party2");
	private static final PartyId PARTY3ID = new PartyId("party3");

	// don't try to mock settings, all parts are needed and mocking
	// makes the test unreadable.
	private final String setstr = "{\"MOPACSettings\":{\"participants\":["
			+ "{\"party\":{\"partyref\":\"http://party1\",\"parameters\":{}},\"profile\":\"http://profile1\"},"
			+ "{\"party\":{\"partyref\":\"http://party2\",\"parameters\":{}},\"profile\":\"http://profile2\"},"
			+ "{\"party\":{\"partyref\":\"http://party3\",\"parameters\":{}},\"profile\":\"http://profile3\"}],"
			+ "\"deadline\":{\"deadlinetime\":{\"durationms\":100}},\"votingevaluator\":{\"LargestAgreement\":{}}}}";

	private MOPACSettings settings;
	private Reporter logger = mock(Reporter.class);
	private MOPAC mopac;

	private final ProtocolToPartyConnFactory factory = mock(
			ProtocolToPartyConnFactory.class);
	private final ProtocolToPartyConn conn1 = mock(ProtocolToPartyConn.class),
			conn2 = mock(ProtocolToPartyConn.class),
			conn3 = mock(ProtocolToPartyConn.class);

	@Before
	public void before() throws JsonParseException, JsonMappingException,
			IOException, NoResourcesNowException {
		settings = jackson.readValue(setstr, MOPACSettings.class);
		mopac = settings.getProtocol(logger);

		List connections = Arrays.asList(conn1, conn2, conn3);
		when(factory.connect(any(List.class))).thenReturn(connections);

		// hack the state.with(connection) stuff simplistically

		when(conn1.getParty()).thenReturn(PARTY1ID);
		when(conn2.getParty()).thenReturn(PARTY2ID);
		when(conn3.getParty()).thenReturn(PARTY3ID);
	}

	@Test
	public void testSmoke() {
	}

	@Test
	public void testGetDescr() {
		assertNotNull(mopac.getDescription());
	}

	@Test
	public void testInitialState() {
		MOPACState state = mopac.getState();
		assertEquals(settings, state.getSettings());
		assertTrue(state.getActions().isEmpty());
	}

	@Test
	public void testProtocol() {
		assertEquals("MOPAC", mopac.getRef().getURI().getPath());
	}

	@Ignore
	@Test
	public void testWorkingCheckDeadline()
			throws IOException, NoResourcesNowException, InterruptedException {

		mopac.start(factory);

		System.out.println(mopac.getState());

		// we have deadline in 100ms so check
		Thread.sleep(200);

		System.out.println(mopac.getState());
		assertTrue(mopac.getState().isFinal(System.currentTimeMillis()));
	}

	@Ignore
	@Test
	public void testSubscribedToConnections() {
		mopac.start(factory);
		verify(conn1, times(1)).addListener(any());
		verify(conn2, times(1)).addListener(any());
		verify(conn3, times(1)).addListener(any());
	}

	@Ignore
	@Test
	public void testActionIsBroadcast()
			throws IOException, NoResourcesNowException, InterruptedException {

		mopac.start(factory);
		Action offer = mock(Offer.class);
		mopac.actionRequest(conn1, offer, 1l);

		// 1 time for YourTurn, one time for ActionDone.
		// but filtering ActionDone here (instead of Action) does not work,
		// maybe bug in Mockito?
		verify(conn1, times(2)).send(any(Inform.class));
	}

	@Test
	public void testAgreementTerminates() {

	}
}
