package geniusweb.protocol.session.mopac;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.junit.Test;

import geniusweb.actions.PartyId;
import geniusweb.inform.Agreements;
import geniusweb.protocol.ProtocolException;

public class PartyStatesTest {

	private final PartyId party1 = new PartyId("party1");
	private final PartyId party2 = new PartyId("party2");
	private final PartyId party3 = new PartyId("party3");

	private final Map<PartyId, Integer> powers = new HashMap<>();
	private final PartyStates states;

	public PartyStatesTest() {
		powers.put(party1, 2);
		powers.put(party2, 3);
		powers.put(party3, 3);
		states = new PartyStates(powers);
	}

	@Test
	public void testBasics() {
		assertEquals(powers.keySet(), states.getNegotiatingParties());
		assertEquals(powers.keySet(), states.getNotYetActed());
		assertEquals(0, states.getExceptions().size());
	}

	@Test
	public void testException() {
		PartyStates newstates = states
				.with(new ProtocolException("bla", party1));
		assertEquals(new Agreements(), newstates.getAgreements());
		assertEquals(powers.keySet(), states.getNegotiatingParties());
		assertEquals(new HashSet<>(Arrays.asList(party2, party3)),
				newstates.getNotYetActed());
	}

	@Test
	public void testFinish() {
		PartyStates newstates = states.finish();
		assertEquals(new Agreements(), newstates.getAgreements());
		assertEquals(powers.keySet(), states.getNegotiatingParties());
		assertEquals(Collections.emptySet(), newstates.getNotYetActed());
	}
}
