package geniusweb.protocol.session.mopac.phase;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;

import geniusweb.actions.EndNegotiation;
import geniusweb.actions.PartyId;
import geniusweb.actions.Votes;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.session.mopac.PartyStates;
import geniusweb.voting.VotingEvaluator;

/**
 * We also test defaultPhase here.
 */
public class OfferPhaseTest {

	private final PartyId party1 = new PartyId("party1");
	private final PartyId party2 = new PartyId("party2");
	private final PartyId party3 = new PartyId("party3");

	private final Map<PartyId, Integer> powers = new HashMap<>();
	private final PartyStates states;
	private final VotingEvaluator evaluator = mock(VotingEvaluator.class);
	private final OfferPhase phase;

	public OfferPhaseTest() {
		powers.put(party1, 2);
		powers.put(party2, 3);
		powers.put(party3, 3);
		states = new PartyStates(powers);
		phase = new OfferPhase(null, states, 10l, evaluator);
	}

	@Test
	public void isInitFinalTest() {
		assertFalse(phase.isFinal(1l));
		assertTrue(phase.isFinal(10l));
	}

	@Test(expected = ProtocolException.class)
	public void checkIncorrectActorTest() throws ProtocolException {
		phase.checkAction(party1, new EndNegotiation(party2), 1);
	}

	@Test(expected = ProtocolException.class)
	public void checkNotAllowedActionTest() throws ProtocolException {
		phase.checkAction(party1, new Votes(party2, Collections.emptyList()),
				1);
	}

	@Test
	public void testException() {
		OfferPhase newphase = phase.with(new ProtocolException("bla", party1));
		assertEquals(10l, newphase.getDeadline());
		assertEquals(0l, phase.getPartyStates().getActions().size());
	}

}
